"""operations with texts up to 1024 tokens (around 500 words)"""
from ..models.text import BaseChatCompletion, GrammarCorrection, TranslateText


def get_answer_with_instruction(question: str, instruction: str) -> str:
    b = BaseChatCompletion(question)
    b.task_for_ai = instruction
    return b.chat_completion_task()


async def aget_answer_with_instruction(question: str, instruction: str) -> str:
    b = BaseChatCompletion(question)
    b.task_for_ai = instruction
    result = await b.async_chat_completion_task()
    return result


def get_answer(question: str) -> str:
    b = BaseChatCompletion(question)
    b.task_for_ai = "You are a helpful assistant"
    b.temperature = 0.5
    result = b.chat_completion_task()
    return result


# print(get_answer('what is internet'))


async def aget_answer(question: str) -> str:
    b = BaseChatCompletion(question)
    b.task_for_ai = "You are a helpful assistant"
    b.temperature = 0.5
    result = await b.async_chat_completion_task()
    return result


def correct_grammar(text: str) -> str:
    g = GrammarCorrection(text)
    result = g.chat_completion_task()
    return result


async def acorrect_grammar(text: str) -> str:
    g = GrammarCorrection(text)
    result = await g.async_chat_completion_task()
    return result


def translate_text_to_Ukrainian(text: str):
    t = TranslateText(text)
    result = t.chat_completion_task()
    return result


# print(translate_text_to_Ukrainian("I don't care"))


async def atranslate_text_to_Ukrainian(text: str):
    t = TranslateText(text)
    result = await t.async_chat_completion_task()
    return result


# add more from text.py!



import os

from dotenv import load_dotenv

# Get the current working directory
current_directory = os.getcwd()
dotenv_path = os.path.join(current_directory, '.env')

load_dotenv(dotenv_path)  # Load variables from .env file

OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")



from .functions.picture import get_picture, aget_picture, get_n_pictures, aget_n_pictures
from .functions.text import get_answer, aget_answer, get_answer_with_instruction, aget_answer_with_instruction
from .functions.text import correct_grammar, acorrect_grammar, translate_text_to_Ukrainian, atranslate_text_to_Ukrainian



import openai

import os
import base64

from .api_key import OPENAI_API_KEY

def normalize_file_name(file_name):
    # Remove leading and trailing whitespace
    file_name = file_name.strip()
    
    # Replace spaces with underscores
    file_name = file_name.replace(' ', '_')
    
    # Remove any remaining characters that are not alphanumeric, underscores, or periods
    file_name = ''.join(c for c in file_name if c.isalnum() or c in ('_',))
    
    # Limit the length of the file name (adjust as needed)
    max_length = 255  # Typical limit for file names on many file systems
    file_name = file_name[:max_length]
    
    return file_name

def pic_saver(b64_json_string, file_name):  # no format
    # Decode the base64 encoded string
    decoded_json_string = base64.b64decode(b64_json_string)
    # Normalize the file name
    normalized_file_name = normalize_file_name(file_name)
    file_path = os.path.join(os.getcwd(), f"{normalized_file_name}.png")
    with open(file_path, "wb") as f:
        f.write(decoded_json_string)


# ------------------------------------------------------Generate Pictures from Text------------------------------------------------------------


class PictureGenerator:
    """
    Generated images can have a size of 256x256, 512x512, or 1024x1024 pixels.
    The more detailed the description, the more likely you are to get the result that you or your end user want.
    Each image can be returned as either a URL or Base64 data, using the response_format parameter.
    1<=n<=10;
    response_format: one of url or b64_json; URLs will expire after an hour.
    """

    def __init__(self, user_input: str, n=1, format="url"):
        self.user_input = user_input
        self.n = n  # length of the output list; amount of options
        self.format = format  #'b64_json'

    def task(self) -> list:
        openai.api_key = OPENAI_API_KEY
        response = openai.Image.create(
            prompt=self.user_input, n=self.n, response_format=self.format
        )
        result = []
        for i in response["data"]:
            result.append(i[self.format])
        return result

    async def async_task(self) -> list:
        openai.api_key = OPENAI_API_KEY
        response = await openai.Image.acreate(
            prompt=self.user_input, n=self.n, response_format=self.format
        )
        result = []
        for i in response["data"]:
            result.append(i[self.format])
        return result



"""Basics that do the job. Each method call equals one API call."""

import openai
import tiktoken

from .api_key import OPENAI_API_KEY

# ------------------------------------------------------General Custom Instructions------------------------------------------------------------
class BaseChatCompletion:
    model = "gpt-3.5-turbo"
    temperature = 0
    max_tokens = 1024

    def __init__(self, user_input: str):
        self.task_for_ai = None
        self.user_input = user_input

    def _get_messages(self):
        """returns a piece of json needed for the api call"""
        return [
            {"role": "system", "content": self.task_for_ai},
            {"role": "user", "content": self.user_input},
        ]

    def chat_completion_task(self) -> str:
        openai.api_key = OPENAI_API_KEY
        response = openai.ChatCompletion.create(
            model=self.model,
            messages=self._get_messages(),
            temperature=self.temperature,
            max_tokens=self.max_tokens,
        )
        return response["choices"][0]["message"]["content"]

    async def async_chat_completion_task(self) -> str:
        openai.api_key = OPENAI_API_KEY
        response = await openai.ChatCompletion.acreate(
            model=self.model,
            messages=self._get_messages(),
            temperature=self.temperature,
            max_tokens=self.max_tokens,
        )
        return response["choices"][0]["message"]["content"]


# ------------------------------------------------------Grammar correction-------------------------------------------------------------
class GrammarCorrection(BaseChatCompletion):
    def __init__(self, user_input: str):
        self.task_for_ai = "You will be provided with statements in some language, and your task is to convert them to standard language fixing all the grammar."
        self.user_input = user_input


# ------------------------------------------------------Translate----------------------------------------------------------------------
class TranslateText(BaseChatCompletion):
    def __init__(self, user_input: str, target_language="Ukrainian"):
        self.task_for_ai = f"You will be provided with a text, and your task is to translate it into {target_language}."
        self.user_input = user_input


# ------------------------------------------------------Changing Tone of Text------------------------------------------------------
class ChangeTone(BaseChatCompletion):
    temperature = 0.4

    def __init__(self, user_input: str):
        self.task_for_ai = (
            "You will be provided with a text, and your task is to change its tone."
        )
        self.user_input = user_input


# ------------------------------------------------------Rephrasing Text------------------------------------------------------
class RephraseText(BaseChatCompletion):
    def __init__(self, user_input: str):
        self.task_for_ai = (
            "You will be provided with a text, and your task is to rephrase it."
        )
        self.user_input = user_input


# ------------------------------------------------------Summarizing Text------------------------------------------------------
class SummarizeText(BaseChatCompletion):
    def __init__(self, user_input: str):
        self.task_for_ai = "You will be provided with a lengthy text, and your task is to summarize it."
        self.user_input = user_input


# ------------------------------------------------------Text Moderation------------------------------------------------------
def num_tokens_from_string(string: str, model_name: str = "gpt-3.5-turbo") -> int:
    """Returns the number of tokens in a text string. Needed to limit up to 8k+ or whatever"""
    encoding = tiktoken.encoding_for_model(model_name)
    num_tokens = len(encoding.encode(string))
    return num_tokens


class IsHarmfulText:  # free API call, no charge
    def __init__(self, user_input: str):
        self.user_input = user_input

    def validation_task(self) -> list:
        openai.api_key = OPENAI_API_KEY
        response = openai.Moderation.create(input=self.user_input)
        result = []
        r = response["results"][0]
        if r["flagged"]:
            for k, v in r["categories"].items():
                if v:
                    result.append(k)
        return result

    async def async_validation_task(self) -> list:
        openai.api_key = OPENAI_API_KEY
        response = await openai.Moderation.acreate(input=self.user_input)
        result = []
        r = response["results"][0]
        if r["flagged"]:
            for k, v in r["categories"].items():
                if v:
                    result.append(k)
        return result



from ..models.picture import PictureGenerator, pic_saver
from typing import List


def get_picture(text: str, save: bool = True) -> str:
    p = PictureGenerator(text, format="b64_json")
    result = p.task()[0]
    if save:
        pic_saver(result, text[:15].replace(" ", "_"))
    return result


# print(get_picture('a gorgeus cat'))


async def aget_picture(text: str, save: bool = True) -> str:
    p = PictureGenerator(text, format="b64_json")
    result = await p.async_task()
    if save:
        pic_saver(result[0], text[:15].replace(" ", "_"))
    return result


# import asyncio
# print(asyncio.run(aget_picture('girl power')))


def get_n_pictures(text: str, n: int = 3, save: bool = True) -> List[str]:
    """up to 10 pics"""
    p = PictureGenerator(text, n=n, format="b64_json")
    result = p.task()
    if save:
        for i in range(len(result)):
            pic_saver(result[i], text[:15].replace(" ", "_") + f"{i+1}")
    return result


# print(get_n_pictures('a gorgeus cat'))


async def aget_n_pictures(text: str, n: int = 3, save: bool = True) -> List[str]:
    """up to 10 pics"""
    p = PictureGenerator(text, n=n, format="b64_json")
    result = await p.async_task()
    if save:
        for i in range(len(result)):
            pic_saver(result[i], text[:15].replace(" ", "_") + f"{i+1}")
    return result


# import asyncio
# print(asyncio.run(aget_n_pictures('a flower')))

# add get pic urls
